@@ -43,8 +43,8 @@ import torch
 from scipy import linalg
 from torch.nn.functional import adaptive_avg_pool2d
 
-from torch_helper.metrics.measure import Metric
-from torch_helper.archs.inception import InceptionV3
+from torchelper.metrics.measure import Metric
+from torchelper.archs.inception import InceptionV3
 
 # try:
 #     from tqdm import tqdm
